module ArrayTransform::Operations

  class CopyColumn
    include ArrayTransform::Callable

    def initialize(
      column_header: nil,
      column_index: nil,
      data:,
      new_column_header: nil
    )
      @column_header = column_header
      @column_index = column_index
      @data = data
      @new_column_header = new_column_header
    end

    attr_reader :data

    def call
      validate!
      copy_column
    end

    private

    attr_reader :column_header, :new_column_header

    def column_index
      @column_index ||= column_header_index
    end

    def column_header_index
      data[0] && data[0].index(column_header)
    end

    def copy_column
      column_index && data.each do |row|
        row << row[column_index]
      end
      new_column_header && data[0][-1] = new_column_header
    end

    def validate!
      if @column_index && @column_header
        raise(ArgumentError, "can only specify one of :column_index or :column_header")
      end
    end
  end

end
