module ArrayTransform::Operations

  class RemoveColumn
    include ArrayTransform::Callable

    def initialize(
      column_header: nil,
      column_index: nil,
      data:
    )
      @column_header = column_header
      @column_index = column_index
      @data = data
    end

    attr_reader :data

    def call
      validate!
      remove_column
    end

    private

    attr_reader :column_header

    def column_index
      @column_index ||= column_header_index
    end

    def column_header_index
      data[0] && data[0].index(column_header)
    end

    def remove_column
      column_index && data.each do |row|
        row.delete_at(column_index)
      end
    end

    def validate!
      if @column_index && @column_header
        raise(ArgumentError, "can only specify one of :column_index or :column_header")
      end
    end
  end

end
