module ArrayTransform::Operations

  class RemoveRows
    include ArrayTransform::Callable

    def initialize(
      cell_test:,
      column_header: nil,
      column_index: nil,
      data:
    )
      @cell_test = cell_test
      @column_header = column_header
      @column_index = column_index
      @data = data
    end

    attr_reader :data

    def call
      validate!
      remove_rows
    end

    private

    attr_reader :cell_test, :column_header

    def column_index
      @column_index ||= column_header_index
    end

    def column_header_index
      data[0] && data[0].index(column_header)
    end

    def remove_rows
      data.each_with_index.map { |row, row_index|
        row_index if cell_test.call(column_index && row[column_index])
      }.compact.sort.reverse.each { |row_index|
        data.delete_at(row_index)
      }
    end

    def validate!
      if @column_index && @column_header
        raise(ArgumentError, "can only specify one of :column_index or :column_header")
      end
    end
  end

end
